package ch.capi.display 
{
	import flash.events.MouseEvent;	
	import flash.geom.Point;	
	import flash.events.Event;	
	import flash.display.DisplayObject;	
	
	/**
	 * Drag manager over a <code>IDragDelimiter</code> object.
	 * 
	 * @author		Cedric Tabin - thecaptain
	 * @version 	1.0
	 */
	public class DragManager 
	{
		//---------//
		//Variables//
		//---------//
		private var _delimiter:IDragDelimiter;
		private var _dragObject:DisplayObject;
		
		private var _dragging:Boolean = false;
		private var _diffX:Number = 0;
		private var _diffY:Number = 0;
		
		/**
		 * Defines the default <code>lockCenter</code> parameter value for calling the
		 * <code>startDrag()</code> method.
		 * 
		 * @see	#startDrag()	startDrag()
		 * @see	#setDragDropBehaviorEnabled()	setDragDropBehaviorEnabled()
		 */
		protected var defaultLockCenter:Boolean = false;
		
		//-----------------//
		//Getters & Setters//
		//-----------------//
		
		/**
		 * Defines the <code>IDragDelimiter</code> that limits the drag.
		 */
		public function get delimiter():IDragDelimiter { return _delimiter; }
		
		/**
		 * Defines the <code>DisplayObject</code> to drag.
		 */
		public function get dragObject():DisplayObject { return _dragObject; }

		//-----------//
		//Constructor//
		//-----------//
		
		/**
		 * Creates a new <code>DragLimitManager</code> object.
		 * 
		 * @param	dragObject		The <code>DisplayObject</code> to drag.
		 * @param	delimiter		The <code>IDragDelimiter</code> object.
		 */
		public function DragManager(dragObject:DisplayObject, delimiter:IDragDelimiter=null):void
		{
			_delimiter = delimiter;
			_dragObject = dragObject;
		}

		//--------------//
		//Public methods//
		//--------------//
		
		/**
		 * Start dragging the <code>DisplayObject</code>. If the <code>DisplayObject</code> is currently being dragged, then
		 * this method hasn't any effect.
		 * 
		 * @param	lockCenter		Defines if the cursor must be on the center point of the <code>DisplayObject</code>.
		 */
		public function startDrag(lockCenter:Boolean=false):void
		{
			if (_dragging) return;
			
			_dragging = true;
			calculateDiff(lockCenter);
			updatePosition();
			_dragObject.addEventListener(Event.ENTER_FRAME, enterFrame, false, 0, true);
		}

		/**
		 * Stop dragging the <code>DisplayObject</code>. If the <code>DisplayObject</code> isn't being currently dragged, then
		 * this method hasn't any effect.
		 */
		public function stopDrag():void
		{
			if (!_dragging) return;
			
			_dragObject.removeEventListener(Event.ENTER_FRAME, enterFrame);
			_dragging = false;
		}
		
		/**
		 * Retrieves if the current <code>DisplayObject</code> is being dragged or not.
		 * 
		 * @return	<code>true</code> if the <code>startDrag()</code> method has been called but not the <code>stopDrag()</code> method.
		 */
		public function isDragging():Boolean
		{
			return _dragging;
		}

		/**
		 * Enables or disables the default drag and drop behavior. This method simply add or remove the listeners to the <code>MouseEvent.MOUSE_DOWN</code>
		 * and <code>MouseEvent.MOUSE_UP</code> events. Note that the <code>DisplayObject</code> must be already placed on the displayed tree !
		 * 
		 * @param	value		If <code>true</code> enableds the default drag and drop behavior.
		 * @param	lockCenter	Defines the value of the <code>lockCenter</code> parameter. If <code>value</code> false, this parameters doesn't change anything.
		 * @see		#dragObject	dragObject
		 */
		public function setDragDropBehaviorEnabled(value:Boolean=true, lockCenter:Boolean=false):void
		{
			if (value)
			{
				defaultLockCenter = lockCenter;
				_dragObject.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown, false, 0, true);
				_dragObject.stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp, false, 0, true);
			}
			else
			{
				_dragObject.removeEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
				_dragObject.stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			}
		}

		//-----------------//
		//Protected methods//
		//-----------------//
		
		/**
		 * Updates the position of the dragged <code>DisplayObject</code>.
		 */
		protected function updatePosition():void
		{
			var pmx:Number = _dragObject.parent.mouseX;
			var pmy:Number = _dragObject.parent.mouseY;
			
			var tx:Number = pmx-_diffX;
			var ty:Number = pmy-_diffY;
			
			var npt:Point = (_delimiter != null) ? _delimiter.getNearestHitPoint(tx, ty) : new Point(tx, ty);
			_dragObject.x = npt.x;
			_dragObject.y = npt.y;
		}

		/**
		 * Starts the drag and drop.
		 * 
		 * @param	evt		The event object.
		 * @see		#startDrag()	startDrag()
		 * @see		#defaultLockCenter	defaultLockCenter
		 */
		protected function onMouseDown(evt:MouseEvent):void
		{
				startDrag(defaultLockCenter);
		}
		
		/**
		 * Stops the drag and drop.
		 * 
		 * @param	evt		The event object.
		 */
		protected function onMouseUp(evt:MouseEvent):void
		{
			stopDrag();
		}

		//---------------//
		//Private methods//
		//---------------//
		
		/**
		 * @private
		 */
		private function enterFrame(evt:Event):void
		{
			updatePosition();
		}
		
		/**
		 * @private
		 */
		private function calculateDiff(lockCenter:Boolean):void
		{
			_diffX = (!lockCenter) ? _dragObject.mouseX : 0;
			_diffY = (!lockCenter) ? _dragObject.mouseY : 0;
		}
	}
}
